from numpy import concatenate, reshape, dot
from scipy import special, optimize, linalg

from helmholtz_mfs import helmholtz, linls

class helmholtz_nonlin(helmholtz):
    # def __init__(self, *args):
    #     helmholtz.__init__(self, *args)
    #     self.nu = 0

    def nonlin_pack(self, z, c):
        p = concatenate((z[0], z[1], c.real, c.imag))
        return p

    def nonlin_unpack(self, p):
        n = len(p)/4
        z = reshape(p[:2*n], (2, n))
        c = p[2*n:3*n] + 1j*p[3*n:]
        return z, c

    def nonlin(self, p):
        z, c = self.nonlin_unpack(p)
        return self.residual(z, c)

    def seplin(self, p):
        z = reshape(p, (2, -1))
        A = self.matrix(z, self.x)
        f = self.rhs(z)
        c, resids, rank, sv = linalg.lstsq(A, f)
        resid = dot(A, c) - f
        if 1: #self.nu != 0:
            r = concatenate((resid.real, resid.imag))
        else:
            r = concatenate((resid.real, resid.imag, 
                             self.nu*c.real, self.nu*c.imag))
        return r

def nonlinls(prob, z0, c0):
    p0 = prob.nonlin_pack(z0, c0)
    leastsq_output = optimize.leastsq(prob.nonlin, p0.copy(), full_output=1)
    p, cov_p, infodict, mesg, ierr = leastsq_output
    print "nonlinls", ierr, mesg
    z, c = prob.nonlin_unpack(p)
    return z, c

def seplinls(prob, z0, c0):
    p0 = reshape(z0, (-1,))
    leastsq_output = optimize.leastsq(prob.seplin, p0.copy(), full_output=1)
    p, cov_p, infodict, mesg, ierr = leastsq_output
    print "seplinls", ierr, mesg
    z = reshape(p, (2, -1))
    c = linls(prob, z)
    return z, c
